﻿using System.Collections;
using UnityEngine;

namespace Assets.FishTank.Fish
{
    public class FishMovement : MonoBehaviour
    {
        private class MovementSpeed
        {
            public static readonly float Float = 0.1f;
            public static readonly float Creep = 0.5f;
            public static readonly float Swim = 1.0f;
            public static readonly float Dart = 2.0f;
        }

        [SerializeField, Min(0)] private int _minTimeBetweenRandomMovements;
        [SerializeField, Min(0)] private int _maxTimeBetweenRandomMovements;
        [SerializeField, Range(0, 1)] private float _chanceToPerformRandomMovement;
        [SerializeField, Range(0, 1)] private float _chanceToStareAtEdge;
        [SerializeField, Range(0, 10)] private float _maxStareAtEdgeTime;

        private Vector2 _movementDirection;
        private float _movementSpeed;
        private float _timeSinceLastRandomMovement;

        public Vector2 MovementDirection => _movementDirection;

        private void OnEnable()
        {
            _movementDirection = RandomMovementDirection();
            _movementSpeed = RandomMovementSpeed();
        }

        private void Update()
        {
            _timeSinceLastRandomMovement += Time.deltaTime;
            transform.position += (Vector3)_movementDirection * _movementSpeed * Time.deltaTime;
        }

        private void OnTriggerExit2D(Collider2D collision)
        {
            if (collision == null) return;
            _movementSpeed = 0.0f;
            StartCoroutine(StareAtEdge());

        }

        private IEnumerator StareAtEdge()
        {
            if (Random.value < _chanceToStareAtEdge)
            {
                yield return new WaitForSeconds(_maxStareAtEdgeTime * Random.value);
            }

            _movementDirection = -_movementDirection;
            _movementSpeed = RandomMovementSpeed();
            _timeSinceLastRandomMovement = 0;
        }

        private void OnTriggerStay2D(Collider2D collision)
        {
          
            if (_timeSinceLastRandomMovement < _minTimeBetweenRandomMovements) return;
            if (_timeSinceLastRandomMovement >= _maxTimeBetweenRandomMovements && Random.value >= _chanceToPerformRandomMovement) return;

            _movementDirection = RandomMovementDirection();
            _movementSpeed = RandomMovementSpeed();
            _timeSinceLastRandomMovement = 0;
        }

        private Vector2 RandomMovementDirection()
        {
            return Random.insideUnitCircle.normalized;
        }

        private float RandomMovementSpeed() 
        { 
            return Random.value switch
            {
                < 0.25f => MovementSpeed.Float,
                < 0.5f => MovementSpeed.Creep,
                < 0.75f => MovementSpeed.Swim,
                < 1.0f => MovementSpeed.Dart,
                _ => 0,
            };
        }
    }
}